import { Plugin, ShallowUnwrapRef, ComponentPublicInstance, InjectionKey } from 'vue';

declare const _default: (props: Record<string, any> & {}) => any;

/**
 * Plugin to install
 */
declare const gitartDialogPlugin: Plugin;

interface IGDialogItem {
    component: ShallowUnwrapRef<{
        new (): ComponentPublicInstance;
    }>;
    id: number;
    props: {
        modelValue: boolean;
    };
    onRemoveHook?: () => void;
}
type DialogAddMethod = <T extends {
    new (): ComponentPublicInstance;
}, P = Omit<InstanceType<T>['$props'], 'modelValue' | 'confirm'>>(component: T, props: P, params?: {
    onRemoveHook?: () => void;
}) => IGDialogItem;
type DialogRemoveMethod = (id: number, closeDelay?: number) => void;
interface IGDialogMethods {
    /**
     * method for adding dialog
     *
     * add first generic type for component to make props type safe. Otherwise, you can pass any props
     * without any type checking.
     *
     * `addDialog<typeof CloneEntityDialog>(CloneEntityDialog, ...)`
     *
     * @example
     * const $dialog = useGDialog()
     * // ..
     * $dialog.addDialog<typeof CloneEntityDialog>(CloneEntityDialog, {
     *  enities,
     * })
     */
    addDialog: DialogAddMethod;
    removeDialog: DialogRemoveMethod;
}
interface IGDialog extends IGDialogMethods {
    dialogs: IGDialogItem[];
}
type IGitartDialogInjectionKey = InjectionKey<IGDialog>;

/**
 * Injection key
 *
 * Provides access to plugin methods and properties using the vue inject method
 *
 * https://gitart-vue-dialog.gitart.org/guide/usage/plugin-usage.html#usage
 *
 * @example Usage
 * const {
 *   dialogs,
 *   removeDialog,
 * } = inject(gitartDialogInjectionKey)!
 */
declare const gitartDialogInjectionKey: IGitartDialogInjectionKey;

declare const useGDialog: () => IGDialog;
/**
 * helper composables for dialogs that confirm something
 * @param component - dialog componen
 * @param _$dialog - dialog controller. If we don't use it inside the setup function, we should pass it manually
 *
 * add first generic type for component to make props type safe. Otherwise, you can pass any props
 * without any type checking.
 *
 * The dialog component should also have modelValue(boolean) prop and emit 'update:modelValue' event.
 * It will be used to close the dialog.
 *
 * @example
 * const confirmIt = useDialogReturnData<typeof ConfirmDialog>(ConfirmDialog)
 * ...
 * const data = await confirmIt({ title: 'Confirm it' }) // data is boolean
 * @returns {Promise<boolean>} - true if the user confirmed the dialog, false if the user canceled the dialog
 */
declare const useDialogConfirm: <T extends new () => ComponentPublicInstance, P = Omit<InstanceType<T>["$props"], "modelValue" | "confirm">>(component: T, _$dialog?: IGDialog) => (props: P) => Promise<boolean>;
/**
 * helper composables for dialogs that return some data

 * @param component - dialog component. Should accept 'confirm' prop and run it on confirm.
 * @param _$dialog - dialog controller. If we don't use it inside the setup function, we should pass it manually
 *
 * Pass first generic type to specify the type of data that the dialog returns. Second generic type is the type of the dialog component.
 *
 * The dialog component should also have modelValue(boolean) prop and emit 'update:modelValue' event.
 * It will be used to close the dialog.
 *
 * @example
 * const confirmIt = useDialogReturnData<number, typeof ConfirmDialog>(ConfirmDialog)
 * ...
 * const data = await confirmIt({ title: 'Confirm it' }) // data is number or null
 * @returns Promise<T | null> - data that the dialog returns or null if the user canceled the dialog
 */
declare const useDialogReturnData: <D, T extends new () => ComponentPublicInstance, P = Omit<InstanceType<T>["$props"], "modelValue" | "confirm">>(component: T, _$dialog?: IGDialog) => (props: P) => Promise<D | null>;

export { type IGDialog, type IGDialogItem, _default as RDialogSpawn, gitartDialogInjectionKey, gitartDialogPlugin, useDialogConfirm, useDialogReturnData, useGDialog };
