import { defineComponent, inject, h, shallowReactive, reactive } from 'vue';

const errorLogger = {
  pluginIsNotInitialized() {
    console.error("The dialog plugin is not initialized.");
  }
};
const gitartDialogInjectionKey = Symbol("GDialog");
const gitartDialogInjectionFallback = {
  dialogs: [],
  addDialog: () => {
    errorLogger.pluginIsNotInitialized();
    return null;
  },
  removeDialog: () => {
    errorLogger.pluginIsNotInitialized();
  }
};

const RDialogSpawn = defineComponent(() => {
  const {
    dialogs,
    removeDialog
  } = inject(gitartDialogInjectionKey, gitartDialogInjectionFallback);
  if (dialogs === gitartDialogInjectionFallback.dialogs) {
    errorLogger.pluginIsNotInitialized();
  }
  const onClose = (id) => {
    removeDialog(id);
  };
  return () => {
    return dialogs.map((dialog) => {
      return h(dialog.component, {
        ...dialog.props,
        "key": dialog.id,
        "onUpdate:modelValue": () => {
          onClose(dialog.id);
        }
      });
    });
  };
});

const gitartDialogPlugin = {
  install: (app, options) => {
    const defaultCloseDelay = options?.closeDelay ?? 500;
    const defaultProps = options?.props ?? {};
    const dialogs = shallowReactive([]);
    const $dialog = {
      dialogs,
      addDialog: (component, props, { onRemoveHook } = {}) => {
        const item = {
          component,
          id: Date.now() + Math.random(),
          props: reactive({
            modelValue: true,
            ...defaultProps,
            ...props
          }),
          onRemoveHook
        };
        dialogs.push(item);
        return item;
      },
      removeDialog: (id, closeDelay) => {
        const dialog = dialogs.find((d) => d.id === id);
        if (!dialog || !dialog.props.modelValue) {
          return;
        }
        dialog.props.modelValue = false;
        if (dialog.onRemoveHook) {
          dialog.onRemoveHook();
        }
        setTimeout(() => {
          dialogs.splice(dialogs.indexOf(dialog), 1);
        }, closeDelay ?? defaultCloseDelay);
      }
    };
    app.provide(gitartDialogInjectionKey, $dialog);
    app.config.globalProperties.$dialog = $dialog;
  }
};

const useGDialog = () => {
  const $dialog = inject(gitartDialogInjectionKey);
  return $dialog || {};
};
const useDialogConfirm = (component, _$dialog) => {
  const $dialog = _$dialog || useGDialog();
  return (props = {}) => {
    return new Promise((resolve) => {
      let confirmed = false;
      $dialog.addDialog(
        component,
        {
          ...props,
          confirm: () => {
            confirmed = true;
            resolve(true);
          }
        },
        {
          onRemoveHook() {
            if (!confirmed) {
              resolve(false);
            }
          }
        }
      );
    });
  };
};
const useDialogReturnData = (component, _$dialog) => {
  const $dialog = _$dialog || useGDialog();
  return (props) => {
    return new Promise((resolve) => {
      let confirmed = false;
      $dialog.addDialog(
        component,
        {
          ...props,
          confirm: (data) => {
            confirmed = true;
            resolve(data);
          }
        },
        {
          onRemoveHook() {
            if (!confirmed) {
              resolve(null);
            }
          }
        }
      );
    });
  };
};

export { RDialogSpawn, gitartDialogInjectionKey, gitartDialogPlugin, useDialogConfirm, useDialogReturnData, useGDialog };
